#include "sysio.h"
#include <avr/io.h>

//for anti prell purposes
volatile uint8_t prell_btn;
volatile uint8_t prell_bcnt;

//header declared variables
uint8_t disp_buf[6];
volatile uint8_t event_flag;
volatile uint8_t esp_stat;
volatile uint8_t esp_buffer[ESP_BUFSIZE];
volatile uint8_t esp_bufptr;

//functions

void btn_on()
{
	//set buttons as inputs
	DDRD &= ~(ROT_A | ROT_B | ROT_C);

	//turn on pull-up resistor for ROT_C (button)
	PORTD |= ROT_C;

	//setup interrupts
	EICRB = (1<<ISC50) | (1<<ISC60);	//any change on INT5 and INT6
	EIMSK = (1<<INT5) | (1<<INT6);		//enable INT5 and INT6
}

void btn_off()
{
	//disable interrupts
	EICRB = 0;
	EIMSK = 0;

	//turn off pull-up resistor
	PORTD &= ~ROT_C;
}

void disp_on()
{
	//set as outputs, EN is driven by TC0 on OCR0A
	DDRB |= (DSP_CLR | DSP_MOSI | DSP_SCLK | DSP_EN);
	DDRD |= DSP_RCK;

	//enable SPI in master
	SPCR = (1<<SPE) | (1<<MSTR) | (1<<DORD);
	SPSR = 1;	//faster transmission

	//set ready to glow
	PORTB |= DSP_CLR;	//clear
	PORTD &= ~DSP_RCK;	//standby

	//init fast PWM on TC0
	TCCR0A = (1<<COM0A1) | (1<<COM0A0) | (1<<WGM00) | (1<<WGM01);
	TCCR0B = (1<<CS00) | (1<<CS02);		//1024 prescaler

	//set brightness to max
	OCR0A = DSP_BRG;
}

void disp_off()
{
	//disable TC0
	TCCR0B = 0;
	TCCR0A = 0;

	//power off
	PORTB &= ~DSP_CLR;	//clear
	PORTD &= ~DSP_RCK;	//standby
	PORTD |= DSP_EN;	//disable drivers

	//disable SPI
	SPCR = 0;
	SPSR = 0;

	//set as inputs
	//set as outputs, EN is driven by TC0 on OCR0A
	DDRB &= ~(DSP_CLR | DSP_MOSI | DSP_SCLK | DSP_EN);
	DDRD &= ~DSP_RCK;
}

void disp_set_brg(uint8_t val)
{
	OCR0A = val;
}

uint8_t disp_get_brg()
{
	return OCR0A;
}

void disp_show()
{
	for (uint8_t i = 0; i<6; i++)
	{
		SPDR = disp_buf[i];
		while(!(SPSR&(1<<SPIF)));
	}

	//send latch clock
	PORTD |= DSP_RCK;
	PORTD &= ~(DSP_RCK);
}

void alarm_on()
{
	DDRC |= ALARM_O;	//set as output
	PORTC &= ~ALARM_O;	//turn off MOSFET
}

void alarm_off()
{
	PORTC &= ~ALARM_O;	//turn off MOSFET
	DDRC &= ~ALARM_O;	//set as input
}

void alarm_val(uint8_t val)
{
	if (val) PORTC |= ALARM_O;
	else PORTC &= ~ALARM_O;
}

void clock_on()
{
	//TC1 CTC mode, 256 prescaler
	TCCR1B = (1<<CS12) | (1<<WGM12);	//256 prescaler = 0.01sec , ctc mode
	OCR1A = 625;		//for 10ms (0.01sec)
	TCNT1 = 0;			//init counter

	//set interrupt
	TIMSK1 |= (1<<OCIE1A);
}

void clock_off()
{
	//disable TC1
	TCCR1B = 0;
	OCR1A = 0;
	TCNT1 = 0;

	//disable interrupt
	TIMSK1 &= ~(1<<OCIE1A);
}

void uart_on()
{
	//set baud rate for 250k
	UBRR1L = 7;
	UCSR1A |= (1<<U2X1); //"faster" tarnsmission

	//null the pointer before an interrupt
	esp_bufptr = 0;

	//enable TX and RX, enable RX interrupt
	UCSR1B = (1 << RXEN1) | (1 << RXCIE1) | (1<<TXEN1);
}

void uart_off()
{
	//disable UART
	UBRR1L = 0;
	UCSR1A =0;

	//disable TX and RX
	UCSR1B = 0;
}

void uart_send(const char * c_str)
{	
	//data counter
	int i = 0;

	while(c_str[i])
	{
		//waitinf to free the output buffer
		while (!(UCSR1A & (1 << UDRE1)));

		//load char to buffer
		UDR1 = c_str[i];
		i++;
	}
}

void uart_send_eeprom(const char * e_c_str)
{
		//data counter
		int i = 0;

		while(e_c_str[i])
		{
			//waitinf to free the output buffer
			while (!(UCSR1A & (1 << UDRE1)));

			//load char to buffer
			UDR1 = eeprom_read_byte(e_c_str + i);
			i++;
		}
}

void esp_on(uint8_t bootmode)
{
	//set ports as output
	DDRC |= ESP_EN;
	DDRD |= ESP_RST | ESP_GPIO0;

	//reset enable, pull down
	PORTD &= ~ESP_RST;		//disable reset

	//select boot mode
	if (bootmode == ESP_MODE_N) PORTD |= ESP_GPIO0;
	else PORTD &= ~ESP_GPIO0;
	
	//disable reset
	PORTD |= ESP_RST;

	//enable module
	PORTC |= ESP_EN;

	//set status
	esp_stat |= 0x80;
}

void esp_off()
{
	//power off module
	PORTC &= ~ESP_EN;

	//set ports as inputs
	DDRC &= ~ESP_EN;
	DDRD &= ~(ESP_RST | ESP_GPIO0);

	//set status
	esp_stat &= 0x7f;
}